# frozen_string_literal: true

require "thor"

module Dmcli
  # CLI
  # Command-line options and commands are set in the CLI class
  class CLI < Thor
    class_options %i[filepath f] => :string

    package_name "dmcli"
    desc "spells", "List spells"
    method_options %i[caster c] => :string, %i[level l] => :number, %i[edition e] => :string

    def spells
      caster = options.caster? ? options.caster : "any"
      level = options.level? ? options.level : "any"
      @filepath = options.filepath? ? options.filepath : "."

      loader = DndDataLoader.new
      puts loader.load_spells(@filepath, "BECMI", caster, level)
    end

    desc "monsters", "List monsters"
    method_options :edition => :string, :level => :number

    def monsters
      puts "Not implemented"
    end

    desc "package", "Package xml"
    method_options %i[output o] => :string

    def package
      xml = XmlExporter.new
      output = options.output? ? options.output : "output.xml"
      filepath = options.filepath? ? options.filepath : "."

      xml.builder(filepath, output)
    end

    desc "version", "dmcli version"
    def version
      puts "dmcli version #{VERSION}"
    end
  end
end
